package geniusweb.bidspace;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import geniusweb.issuevalue.Bid;
import geniusweb.profile.utilityspace.LinearAdditiveUtilitySpace;
import tudelft.utilities.immutablelist.AbstractImmutableList;

/**
 * Object containing a subset of all bids, that have utility within given range.
 */
public class BidsWithUtility extends AbstractImmutableList<Bid> {

	private final List<Bid> collectedbids;

	/**
	 * Create Bids with utility within given bounds.
	 * 
	 * @param space the {@link LinearAdditiveUtilitySpace} to search
	 * @param min   the minimum required utility
	 * @param max   the maximum required utility
	 */
	public BidsWithUtility(LinearAdditiveUtilitySpace space, BigDecimal min, BigDecimal max) {
		this.collectedbids = new ArrayList<Bid>(getBids(space, min, max));
	}

	/**
	 * Brute force hack for this time. Maybe we can construct this in a lazy way.
	 * 
	 * @param space
	 * @param min
	 * @param max
	 * @return
	 */
	private List<Bid> getBids(LinearAdditiveUtilitySpace space, BigDecimal min, BigDecimal max) {
		List<Bid> goodbids = new LinkedList<Bid>();
		for (Bid bid : new AllBidsList(space.getDomain())) {
			BigDecimal util = space.getUtility(bid);
			if (util.compareTo(min) >= 0 && util.compareTo(max) <= 0) {
				goodbids.add(bid);
			}
		}
		return goodbids;
	}

	@Override
	public Bid get(BigInteger index) {
		return collectedbids.get(index.intValue());
	}

	@Override
	public BigInteger size() {
		return BigInteger.valueOf(collectedbids.size());
	}

}
